/// <reference types="node" />
import { EventEmitter } from 'events';
import { UmzugStorage } from './storage';
export declare type Promisable<T> = T | PromiseLike<T>;
export declare type LogFn = (message: string) => void;
/** Constructor options for the Umzug class */
export interface UmzugOptions<Ctx = never> {
    /** The migrations that the Umzug instance should perform */
    migrations: InputMigrations<Ctx>;
    /** A logging function. Pass `console` to use stdout, or pass in your own logger. Pass `undefined` explicitly to disable logging. */
    logger: Record<'info' | 'warn' | 'error', LogFn> | undefined;
    /** The storage implementation. By default, `JSONStorage` will be used */
    storage?: UmzugStorage;
    /** An optional context object, which will be passed to each migration function, if defined */
    context?: Ctx;
}
/** Serializeable metadata for a migration. The structure returned by the external-facing `pending()` and `executed()` methods. */
export interface MigrationMeta {
    /** Name - this is used as the migration unique identifier within storage */
    name: string;
    /** An optional filepath for the migration. Note: this may be undefined, since not all migrations correspond to files on the filesystem */
    path?: string;
}
/**
 * A runnable migration. Represents a migration object with an `up` function which can be called directly, with no arguments, and an optional `down` function to revert it.
 */
export interface RunnableMigration extends MigrationMeta {
    /** The effect of applying the migration */
    up: () => Promise<unknown>;
    /** The effect of reverting the migration */
    down?: () => Promise<unknown>;
}
/**
 * Allowable inputs for migrations. Can be either glob instructions for migration files, a list of runnable migrations, or a
 * function which receives a context and returns a list of migrations.
 */
export declare type InputMigrations<T> = {
    /**
     * A glob string for migration files. Can also be in the format `[path/to/migrations/*.js', {cwd: 'some/base/dir', ignore: '**ignoreme.js' }]`
     * See https://npmjs.com/package/glob for more details on the glob format - this package is used internally.
     */
    glob: string | [string, {
        cwd?: string;
        ignore?: string | string[];
    }];
    /** Will be supplied to every migration function. Can be a database client, for example */
    context?: T;
    /** A function which takes a migration name, path and context, and returns an object with `up` and `down` functions */
    resolve?: Resolver<T>;
} | RunnableMigration[] | ((context: T) => Promisable<RunnableMigration[]>);
/** A function which takes a migration name, path and context, and returns an object with `up` and `down` functions */
export declare type Resolver<T> = (params: {
    path: string;
    name: string;
    context: T;
}) => RunnableMigration;
export declare const RerunBehavior: {
    /** Hard error if an up migration that has already been run, or a down migration that hasn't, is encountered */
    readonly THROW: "THROW";
    /** Silently skip up migrations that have already been run, or down migrations that haven't */
    readonly SKIP: "SKIP";
    /** Re-run up migrations that have already been run, or down migrations that haven't */
    readonly ALLOW: "ALLOW";
};
export declare type RerunBehavior = keyof typeof RerunBehavior;
export declare type MigrateUpOptions = {
    /** If specified, migrations up to and including this name will be run. Otherwise, all pending migrations will be run */
    to?: string;
    /** Should not be specified with `to` */
    migrations?: never;
    /** Should not be specified with `to` */
    rerun?: never;
} | {
    /** If specified, only the migrations with these names migrations will be run. An error will be thrown if any of the names are not found in the list of available migrations */
    migrations: string[];
    /** What to do if a migration that has already been run is explicitly specified. Default is `THROW`. */
    rerun?: RerunBehavior;
    /** Should not be specified with `migrations` */
    to?: never;
};
export declare type MigrateDownOptions = {
    /** If specified, migrations down to and including this name will be revert. Otherwise, only the last executed will be reverted */
    to?: string | 0;
    /** Should not be specified with `to` */
    migrations?: never;
    /** Should not be specified with `to` */
    rerun?: never;
} | {
    /**
     * If specified, only the migrations with these names migrations will be reverted. An error will be thrown if any of the names are not found in the list of executed migrations.
     * Note, migrations will be run in the order specified.
     */
    migrations: string[];
    /** What to do if a migration that has not been run is explicitly specified. Default is `THROW`. */
    rerun?: RerunBehavior;
    /** Should not be specified with `migrations` */
    to?: never;
};
export declare class Umzug<Ctx> extends EventEmitter {
    private readonly options;
    private readonly storage;
    private readonly migrations;
    /**
     * Compile-time only property for type inference. After creating an Umzug instance, it can be used as type alias for
     * a user-defined migration. The function receives a migration name, path and the context for an umzug instance
     * @example
     * // migrator.ts
     * import { Umzug } from 'umzug'
     *
     * const umzug = new Umzug({...})
     * export type Migration = typeof umzug._types.migration;
     *
     * umzug.up();
     *
     * @example
     * // migration-1.ts
     * import type { Migration } from '../migrator'
     *
     * // name and context will now be strongly-typed
     * export const up: Migration = ({name, context}) => context.query(...)
     * export const down: Migration = ({name, context}) => context.query(...)
     */
    readonly _types: {
        migration: (params: {
            name: string;
            path?: string;
            context: Ctx;
        }) => Promise<unknown>;
    };
    /** creates a new Umzug instance */
    constructor(options: UmzugOptions<Ctx>);
    private logging;
    static defaultResolver: Resolver<unknown>;
    /**
     * create a clone of the current Umzug instance, allowing customising the list of migrations.
     * This could be used, for example, to sort the list of migrations in a specific order.
     */
    extend(transform: (migrations: readonly RunnableMigration[]) => Promisable<RunnableMigration[]>): Umzug<Ctx>;
    /** Get the list of migrations which have already been applied */
    executed(): Promise<MigrationMeta[]>;
    /** Get the list of migrations which have already been applied */
    private _executed;
    /** Get the list of migrations which are yet to be applied */
    pending(): Promise<MigrationMeta[]>;
    private _pending;
    /**
     * Apply migrations. By default, runs all pending migrations.
     * @see MigrateUpOptions for other use cases using `to`, `migrations` and `rerun`.
     */
    up(options?: MigrateUpOptions): Promise<void>;
    /**
     * Revert migrations. By default, the last executed migration is reverted.
     * @see MigrateDownOptions for other use cases using `to`, `migrations` and `rerun`.
     */
    down(options?: MigrateDownOptions): Promise<void>;
    private findNameIndex;
    private findMigrations;
    /** helper for parsing input migrations into a callback returning a list of ready-to-run migrations */
    private getMigrationsResolver;
}
