"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Umzug = exports.RerunBehavior = void 0;
const path = require("path");
const events_1 = require("events");
const util_1 = require("util");
const storage_1 = require("./storage");
const glob = require("glob");
const globAsync = util_1.promisify(glob);
exports.RerunBehavior = {
    /** Hard error if an up migration that has already been run, or a down migration that hasn't, is encountered */
    THROW: 'THROW',
    /** Silently skip up migrations that have already been run, or down migrations that haven't */
    SKIP: 'SKIP',
    /** Re-run up migrations that have already been run, or down migrations that haven't */
    ALLOW: 'ALLOW',
};
class Umzug extends events_1.EventEmitter {
    /** creates a new Umzug instance */
    constructor(options) {
        super();
        this.options = options;
        this.storage = storage_1.verifyUmzugStorage(options.storage || new storage_1.JSONStorage());
        this.migrations = this.getMigrationsResolver();
    }
    logging(message) {
        var _a;
        (_a = this.options.logger) === null || _a === void 0 ? void 0 : _a.info(message);
    }
    /**
     * create a clone of the current Umzug instance, allowing customising the list of migrations.
     * This could be used, for example, to sort the list of migrations in a specific order.
     */
    extend(transform) {
        return new Umzug({
            ...this.options,
            migrations: async () => {
                const migrations = await this.migrations();
                return transform(migrations);
            },
        });
    }
    /** Get the list of migrations which have already been applied */
    async executed() {
        const list = await this._executed();
        // We do the following to not expose the `up` and `down` functions to the user
        return list.map(m => ({ name: m.name, path: m.path }));
    }
    /** Get the list of migrations which have already been applied */
    async _executed() {
        const [migrations, executedNames] = await Promise.all([this.migrations(), this.storage.executed()]);
        const executedSet = new Set(executedNames);
        return migrations.filter(m => executedSet.has(m.name));
    }
    /** Get the list of migrations which are yet to be applied */
    async pending() {
        const list = await this._pending();
        // We do the following to not expose the `up` and `down` functions to the user
        return list.map(m => ({ name: m.name, path: m.path }));
    }
    async _pending() {
        const [migrations, executedNames] = await Promise.all([this.migrations(), this.storage.executed()]);
        const executedSet = new Set(executedNames);
        return migrations.filter(m => !executedSet.has(m.name));
    }
    /**
     * Apply migrations. By default, runs all pending migrations.
     * @see MigrateUpOptions for other use cases using `to`, `migrations` and `rerun`.
     */
    async up(options = {}) {
        const eligibleMigrations = async () => {
            if (options.migrations && options.rerun === exports.RerunBehavior.ALLOW) {
                // Allow rerun means the specified migrations should be run even if they've run before - so get all migrations, not just pending
                const list = await this.migrations();
                return this.findMigrations(list, options.migrations);
            }
            if (options.migrations && options.rerun === exports.RerunBehavior.SKIP) {
                const executedNames = new Set((await this._executed()).map(m => m.name));
                const filteredMigrations = options.migrations.filter(m => !executedNames.has(m));
                return this.findMigrations(await this.migrations(), filteredMigrations);
            }
            if (options.migrations) {
                return this.findMigrations(await this._pending(), options.migrations);
            }
            const allPending = await this._pending();
            let sliceIndex = allPending.length;
            if (options.to) {
                sliceIndex = this.findNameIndex(allPending, options.to) + 1;
            }
            return allPending.slice(0, sliceIndex);
        };
        const toBeApplied = await eligibleMigrations();
        for (const m of toBeApplied) {
            const start = Date.now();
            this.logging('== ' + m.name + ': migrating =======');
            this.emit('migrating', m.name, m);
            await m.up();
            await this.storage.logMigration(m.name);
            const duration = ((Date.now() - start) / 1000).toFixed(3);
            this.logging(`== ${m.name}: migrated (${duration}s)\n`);
            this.emit('migrated', m.name, m);
        }
    }
    /**
     * Revert migrations. By default, the last executed migration is reverted.
     * @see MigrateDownOptions for other use cases using `to`, `migrations` and `rerun`.
     */
    async down(options = {}) {
        var _a;
        const eligibleMigrations = async () => {
            if (options.migrations && options.rerun === exports.RerunBehavior.ALLOW) {
                const list = await this.migrations();
                return this.findMigrations(list, options.migrations);
            }
            if (options.migrations && options.rerun === exports.RerunBehavior.SKIP) {
                const pendingNames = new Set((await this._pending()).map(m => m.name));
                const filteredMigrations = options.migrations.filter(m => !pendingNames.has(m));
                return this.findMigrations(await this.migrations(), filteredMigrations);
            }
            if (options.migrations) {
                return this.findMigrations(await this._executed(), options.migrations);
            }
            const executedReversed = (await this._executed()).slice().reverse();
            let sliceIndex = 1;
            if (options.to === 0 || options.migrations) {
                sliceIndex = executedReversed.length;
            }
            else if (options.to) {
                sliceIndex = this.findNameIndex(executedReversed, options.to) + 1;
            }
            return executedReversed.slice(0, sliceIndex);
        };
        const toBeReverted = await eligibleMigrations();
        for (const m of toBeReverted) {
            const start = Date.now();
            this.logging('== ' + m.name + ': reverting =======');
            this.emit('reverting', m.name, m);
            await ((_a = m.down) === null || _a === void 0 ? void 0 : _a.call(m));
            await this.storage.unlogMigration(m.name);
            const duration = ((Date.now() - start) / 1000).toFixed(3);
            this.logging(`== ${m.name}: reverted (${duration}s)\n`);
            this.emit('reverted', m.name, m);
        }
    }
    findNameIndex(migrations, name) {
        const index = migrations.findIndex(m => m.name === name);
        if (index === -1) {
            throw new Error(`Couldn't find migration to apply with name ${JSON.stringify(name)}`);
        }
        return index;
    }
    findMigrations(migrations, names) {
        const map = new Map(migrations.map(m => [m.name, m]));
        return names.map(name => {
            const migration = map.get(name);
            if (!migration) {
                throw new Error(`Couldn't find migration to apply with name ${JSON.stringify(name)}`);
            }
            return migration;
        });
    }
    /** helper for parsing input migrations into a callback returning a list of ready-to-run migrations */
    getMigrationsResolver() {
        const { migrations: inputMigrations, context } = this.options;
        if (Array.isArray(inputMigrations)) {
            return async () => inputMigrations;
        }
        if (typeof inputMigrations === 'function') {
            return async () => inputMigrations(context);
        }
        const fileGlob = inputMigrations.glob;
        const [globString, globOptions] = Array.isArray(fileGlob) ? fileGlob : [fileGlob];
        const resolver = inputMigrations.resolve || Umzug.defaultResolver;
        return async () => {
            const paths = await globAsync(globString, { ...globOptions, absolute: true });
            return paths.map(unresolvedPath => {
                const filepath = path.resolve(unresolvedPath);
                const name = path.basename(filepath, path.extname(filepath));
                return {
                    name,
                    path: filepath,
                    ...resolver({ context, path: filepath, name }),
                };
            });
        };
    }
}
exports.Umzug = Umzug;
Umzug.defaultResolver = ({ path: filepath, name, context }) => {
    const ext = path.extname(filepath);
    const canRequire = ext === '.js' || ext in require.extensions;
    const languageSpecificHelp = {
        '.ts': "You can use the default resolver with typescript files by adding `ts-node` as a dependency and calling `require('ts-node/register')` at the program entrypoint before running migrations.",
        '.sql': 'Try writing a resolver which reads file content and executes it as a sql query.',
    };
    if (!canRequire) {
        const errorParts = [
            `No resolver specified for file ${filepath}.`,
            languageSpecificHelp[ext],
            `See docs for guidance on how to write a custom resolver.`,
        ];
        throw new Error(errorParts.filter(Boolean).join(' '));
    }
    const getModule = () => require(filepath);
    return {
        name,
        path: filepath,
        up: async () => getModule().up({ path: filepath, name, context }),
        down: async () => getModule().down({ path: filepath, name, context }),
    };
};
//# sourceMappingURL=umzug.js.map